/*
 * Copyright (c) 2003-2006 Alexandre Ratchov <alex@caoua.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 	- Redistributions of source code must retain the above
 * 	  copyright notice, this list of conditions and the
 * 	  following disclaimer.
 *
 * 	- Redistributions in binary form must reproduce the above
 * 	  copyright notice, this list of conditions and the
 * 	  following disclaimer in the documentation and/or other
 * 	  materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef PICPROG_PIC_H
#define PICPROG_PIC_H

/*
 * bit mask of pins we're using
 */
#define PIC_DATA		1	/* RB7 pin */		
#define PIC_CLOCK		2	/* RB6 pin */

/*
 * 6bit commands used by the chip, see datasheet
 */
#define PIC_LOADCONF		0	/* load data for configuration */
#define PIC_LOADPROG		2	/* load data for program memory */
#define PIC_READPROG		4	/* read data from program memory */
#define PIC_INCRADDR		6	/* increment address */

#define PIC_LOADDATA		3	/* load data for data memory */
#define	PIC_READDATA		5	/* read data from data memory */
#define PIC_BEGINERASEWRITE	8	/* begin erase & programming cycle */
#define PIC_BEGINWRITE		0x18	/* begin programming only cycle */
#define PIC_ERASEPROG		0x9	/* bulk erase program memory */
#define PIC_ERASEDATA		0xb	/* bulk erase data memory */

struct pic {
	int fd;		/* file desc. of tty(4) device where the programmer is plugged */
	int lines;	/* state of modem lines of the tty(4) device */
};


/*
 * the following are implemented in tty.c
 */
unsigned pic_init(struct pic *, char *);
void pic_done(struct pic *);
void pic_reset(struct pic *);
void pic_program(struct pic *);
void pic_set(struct pic *, unsigned);
unsigned pic_get(struct pic *);
void pic_delay(struct pic *);

/*
 * the following are implemented in pic.c
 */
void pic_command(struct pic *, unsigned);
void pic_writeword(struct pic *, unsigned);
unsigned pic_readword(struct pic *);

void pic_dataread(struct pic *pic, unsigned char *data, unsigned size);
void pic_datawrite(struct pic *pic, unsigned char *data, unsigned size);
void pic_dataerase(struct pic *pic);
void pic_progread(struct pic *pic, unsigned char *data, unsigned size);
void pic_progwrite(struct pic *pic, unsigned char *data, unsigned size);
void pic_progerase(struct pic *pic);
void pic_cfwrite(struct pic *pic, unsigned char *data, unsigned size);
void pic_cfread(struct pic *pic, unsigned char *data, unsigned size);
void pic_unprotect(struct pic *pic);

#endif
