;
; sample program for PIC 16F84A microcontroller:
; make blink a LED connected on port A0
;
; to compile it:
;
;	gpasm -a inhx8m -p pic16f84a sample.asm
;
; to burn it:
;
;	picprog -f /dev/cua01 -w sample.hex
;


;
; bank 0 spectial registers
;
STATUS		equ	0x03	
PORTA		equ	0x05
PORTB		equ	0x06

;
; bank 1 spectial registers
;
TRISA		equ	0x05
		
;
; status register bits
;
RP0		equ	5
TO		equ	4
PD		equ	3
Z		equ	2
DC		equ	1
C		equ	0
		
;
; direction bits
;
W		equ	0
F		equ	1

; ----------------------------------------------------------------------
	
;
; general purpose registers we use (ie our variables)
;
cnt_hi		equ	0x0c
cnt_lo		equ	0x0d

; ----------------------------------------------------------------------

		;
		; entry point
		;
		org	0

		;
		; clear all bits on PORTA
		;
		clrf	PORTA

		;
		; select "output" direction for all PORTA bits
		;
		bsf	STATUS, RP0
		clrf	TRISA
		bcf	STATUS, RP0

loop
		;
		; toggle PORTA, bit 0
		;
		movlw	0x1
		xorwf	PORTA, F

		;
		; sleep
		;
		movlw	0xf0
		call	delay
		
		goto	loop


;
; sleep
;
delay		movwf	cnt_hi
		clrf	cnt_lo

delay_loop	decfsz	cnt_lo, F
		goto	delay_loop
		decfsz	cnt_hi, F
		goto	delay_loop
		return			

; ----------------------------------------------------------------------

;
; oscillator type bits in the config word
;
PROTECT_OFF	equ	0x3ff0
PWRTE_OFF	equ	8
WTD_ON		equ	4
OSC_RC		equ	3
OSC_HS		equ	2
OSC_XT		equ	1
OSC_LP		equ	0
	
		;
		; configuration word is at 2007
		;
		org	0x2007
		dw	PROTECT_OFF | PWRTE_OFF | OSC_XT

		end
