/*
 * Copyright (c) 2003-2006 Alexandre Ratchov <alex@caoua.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 	- Redistributions of source code must retain the above
 * 	  copyright notice, this list of conditions and the
 * 	  following disclaimer.
 *
 * 	- Redistributions in binary form must reproduce the above
 * 	  copyright notice, this list of conditions and the
 * 	  following disclaimer in the documentation and/or other
 * 	  materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 *	tty.c - low level interface using bsd/linux kernel tty(4) driver
 */
#include <sys/ioctl.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include "pic.h"

unsigned
pic_init(struct pic *pic, char *devname)
{
	/*
	 * by default /dev/cuaXX is put in raw mode by the kernel
	 * when it is opened, so we just need to open the device and
	 * read "lines" bits (for future use)
	 */
	pic->fd = open(devname, O_RDWR);
	if (pic->fd < 0) {
		perror(devname);
		return 0;
	}
	if (ioctl(pic->fd, TIOCMGET, &pic->lines) < 0) {
		perror("TIOCMGET:");
		exit(1);
	}
	return 1;
}

void
pic_done(struct pic *pic)
{
	pic_reset(pic);
	(void)close(pic->fd);
}

void
pic_set(struct pic *pic, unsigned bits)
{
	unsigned i;

	pic->lines |= (TIOCM_DTR | TIOCM_RTS);
	if (bits & PIC_CLOCK) {
		pic->lines &= ~TIOCM_RTS;
	}
	if (bits & PIC_DATA) {
		pic->lines &= ~TIOCM_DTR;
	}
	if (ioctl(pic->fd, TIOCMSET, &pic->lines) < 0) {
		perror("TIOCMSET:");
		exit(1);
	}
#if 1
	for (i = 0; i < 0x100; i++);
#endif
}

unsigned
pic_get(struct pic *pic)
{
	if (ioctl(pic->fd, TIOCMGET, &pic->lines) < 0) {
		perror("TIOCMGET:");
		exit(1);
	}
	return (pic->lines & TIOCM_CTS) ? 0 : 1;
}

void
pic_delay(struct pic *pic)
{
	usleep(10000);		/* sleep 10ms */
}

void 
pic_program(struct pic *pic)
{
	pic_set(pic, 0);
	if (ioctl(pic->fd, TIOCSBRK) < 0) {
		perror("TIOCCBRK:");
		exit(1);
	}
	pic_delay(pic);
}

void
pic_reset(struct pic *pic)
{
	pic_set(pic, 0);
	if (ioctl(pic->fd, TIOCCBRK) < 0) {
		perror("TIOCSBRK:");
		exit(1);
	}
	pic_delay(pic);
}

